@@ -6,6 +6,21 @@
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE OverloadedStrings #-}
+-- | Provides a high-level interface for processing YAML files.
+--
+-- This module reuses most of the infrastructure from the @aeson@ package.
+-- This means that you can use all of the existing tools for JSON
+-- processing for processing YAML files. As a result, much of the
+-- documentation below mentions JSON; do not let that confuse you, it's
+-- intentional.
+--
+-- For the most part, YAML content translates directly into JSON, and
+-- therefore there is very little data loss. If you need to deal with YAML
+-- more directly (e.g., directly deal with aliases), you should use the
+-- "Text.Libyaml" module instead.
+--
+-- For documentation on the @aeson@ types, functions, classes, and
+-- operators, please see the @Data.Aeson@ module of the @aeson@ package.
 module Data.Yaml
     ( -- * Types
       Value (..)
@@ -99,15 +114,20 @@ objToEvents' (Array list) rest =
 objToEvents' (Object pairs) rest =
     EventMappingStart Nothing
   : foldr ($) (EventMappingEnd : rest) (map pairToEvents $ M.toList pairs)
-objToEvents' (String s) rest = EventScalar (encodeUtf8 s) NoTag Any Nothing : rest
-objToEvents' Null rest = EventScalar "null" NoTag Literal Nothing : rest
-objToEvents' (Bool True) rest = EventScalar "true" NoTag Literal Nothing : rest
-objToEvents' (Bool False) rest = EventScalar "false" NoTag Literal Nothing : rest
-objToEvents' (Number n) rest = EventScalar (S8.pack $ show n) NoTag Literal Nothing : rest
+
+-- Empty strings need special handling to ensure they get quoted. This avoids:
+-- https://github.com/snoyberg/yaml/issues/24
+objToEvents' (String "") rest = EventScalar "" NoTag SingleQuoted Nothing : rest
+
+objToEvents' (String s) rest = EventScalar (encodeUtf8 s) StrTag PlainNoTag Nothing : rest
+objToEvents' Null rest = EventScalar "null" NullTag PlainNoTag Nothing : rest
+objToEvents' (Bool True) rest = EventScalar "true" BoolTag PlainNoTag Nothing : rest
+objToEvents' (Bool False) rest = EventScalar "false" BoolTag PlainNoTag Nothing : rest
+objToEvents' (Number n) rest = EventScalar (S8.pack $ show n) IntTag PlainNoTag Nothing : rest
 
 pairToEvents :: Pair -> [Y.Event] -> [Y.Event]
 pairToEvents (k, v) rest =
-    EventScalar (encodeUtf8 k) NoTag Any Nothing
+    EventScalar (encodeUtf8 k) StrTag PlainNoTag Nothing
   : objToEvents' v rest
 
 -- Parsing
@@ -167,8 +187,8 @@ textToValue SingleQuoted _ t = String t
 textToValue DoubleQuoted _ t = String t
 textToValue _ StrTag t = String t
 textToValue Folded _ t = String t
-textToValue _ _ "null" = Null
 textToValue _ _ t
+    | t `elem` ["null", "Null", "NULL", "~", ""] = Null
     | any (t `isLike`) ["y", "yes", "on", "true"] = Bool True
     | any (t `isLike`) ["n", "no", "off", "false"] = Bool False
     | Right (x, "") <- signed decimal t = Number $ I x
