@@ -8,6 +8,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE TypeFamilies #-}
 
 -- | Low-level, streaming YAML interface. For a higher-level interface, see
 -- "Data.Yaml".
@@ -67,6 +68,7 @@ data Style = Any
            | DoubleQuoted
            | Literal
            | Folded
+           | PlainNoTag
     deriving (Show, Read, Eq, Enum, Bounded, Ord, Data, Typeable)
 
 data Tag = StrTag
@@ -378,13 +380,17 @@ toEventRaw e f = allocaBytes eventSize $ \er -> do
             c_simple_document_start er
         EventDocumentEnd ->
             c_yaml_document_end_event_initialize er 1
-        EventScalar bs thetag style anchor -> do
+        EventScalar bs thetag style0 anchor -> do
             BU.unsafeUseAsCStringLen bs $ \(value, len) -> do
                 let value' = castPtr value :: Ptr CUChar
                     len' = fromIntegral len :: CInt
                 let thetag' = tagToString thetag
                 withCString thetag' $ \tag' -> do
-                    let style' = toEnum $ fromEnum style
+                    let (pi, style) =
+                            case style0 of
+                                PlainNoTag -> (1, Plain)
+                                x -> (0, x)
+                        style' = toEnum $ fromEnum style
                         tagP = castPtr tag'
                         qi = if null thetag' then 1 else 0
                     case anchor of
@@ -395,7 +401,7 @@ toEventRaw e f = allocaBytes eventSize $ \er -> do
                                 tagP    -- tag
                                 value'  -- value
                                 len'    -- length
-                                0       -- plain_implicit
+                                pi      -- plain_implicit
                                 qi      -- quoted_implicit
                                 style'  -- style
                         Just anchor' ->
@@ -459,7 +465,12 @@ newtype ToEventRawException = ToEventRawException CInt
     deriving (Show, Typeable)
 instance Exception ToEventRawException
 
-decode :: MonadResource m => B.ByteString -> GSource m Event
+decode :: MonadResource m => B.ByteString
+#if MIN_VERSION_conduit(1, 0, 0)
+       -> Producer m Event
+#else
+       -> GSource m Event
+#endif
 decode bs | B8.null bs = return ()
 decode bs =
     bracketP alloc cleanup (runParser . fst)
@@ -483,7 +494,12 @@ decode bs =
         c_yaml_parser_delete ptr
         free ptr
 
-decodeFile :: MonadResource m => FilePath -> GSource m Event
+decodeFile :: MonadResource m => FilePath
+#if MIN_VERSION_conduit(1, 0, 0)
+           -> Producer m Event
+#else
+           -> GSource m Event
+#endif
 decodeFile file =
     bracketP alloc cleanup (runParser . fst)
   where
@@ -514,7 +530,12 @@ decodeFile file =
         c_yaml_parser_delete ptr
         free ptr
 
-runParser :: MonadResource m => Parser -> GSource m Event
+runParser :: MonadResource m => Parser
+#if MIN_VERSION_conduit(1, 0, 0)
+           -> Producer m Event
+#else
+           -> GSource m Event
+#endif
 runParser parser = do
     e <- liftIO $ parserParseOne' parser
     case e of
@@ -538,7 +559,12 @@ parserParseOne' parser = allocaBytes eventSize $ \er -> do
           return $ Left $ YamlParseException problem context problemMark
         else Right <$> getEvent er
 
-encode :: MonadResource m => GSink Event m ByteString
+encode :: MonadResource m
+#if MIN_VERSION_conduit(1, 0, 0)
+       => Consumer Event m ByteString
+#else
+       => GSink Event m ByteString
+#endif
 encode =
     runEmitter alloc close
   where
@@ -555,7 +581,11 @@ encode =
 
 encodeFile :: MonadResource m
            => FilePath
+#if MIN_VERSION_conduit(1, 0, 0)
+           -> Consumer Event m ()
+#else
            -> GInfSink Event m
+#endif
 encodeFile filePath =
     bracketP getFile c_fclose $ \file -> runEmitter (alloc file) (\u _ -> return u)
   where
@@ -571,8 +601,13 @@ encodeFile filePath =
 
 runEmitter :: MonadResource m
            => (Emitter -> IO a) -- ^ alloc
+#if MIN_VERSION_conduit(1, 0, 0)
+           -> (() -> a -> IO b) -- ^ close
+           -> Consumer Event m b
+#else
            -> (u -> a -> IO b) -- ^ close
            -> Pipe l Event o u m b
+#endif
 runEmitter allocI closeI =
     bracketP alloc cleanup go
   where
@@ -589,7 +624,12 @@ runEmitter allocI closeI =
     go (emitter, a) =
         loop
       where
+#if MIN_VERSION_conduit(1, 0, 0)
+        loop = await >>= maybe (close ()) push
+#else
         loop = awaitE >>= either close push
+#endif
+
         push e = do
             _ <- liftIO $ toEventRaw e $ c_yaml_emitter_emit emitter
             loop
